"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.shardSchema = exports.shardMetadataSchema = exports.parseShardMetadata = exports.parseShard = exports.parseOutcomes = exports.parseOutcome = exports.parseLocalMetadata = exports.parseExecutionSummary = exports.parseContract = exports.parseCalls = exports.parseCall = exports.parseBlock = exports.outcomesSchema = exports.outcomeSchema = exports.localMetadataSchema = exports.executionSummarySchema = exports.contractSchema = exports.callsSchema = exports.callSchema = exports.buildShardMetadata = exports.buildShard = exports.buildOutcomes = exports.buildOutcome = exports.buildLocalMetadata = exports.buildExecutionSummary = exports.buildContract = exports.buildCalls = exports.buildCall = exports.buildBlock = exports.blockSchema = exports.ShardMetadata = exports.Shard = exports.Outcomes = exports.Outcome = exports.LocalMetadata = exports.ExecutionSummary = exports.Contract = exports.Calls = exports.Call = exports.Block = void 0;

var _borsh = require("borsh");

var _util = require("@skyekiwi/util");

// Copyright 2021-2022 @skyekiwi/s-contract authors & contributors
// SPDX-License-Identifier: Apache-2.0

/* eslint-disable sort-keys, camelcase */
class Call {
  constructor(config) {
    this.origin = void 0;
    this.origin_public_key = void 0;
    this.encrypted_egress = void 0;
    this.transaction_action = void 0;
    this.receiver = void 0;
    this.amount = void 0;
    this.wasm_blob = void 0;
    this.method = void 0;
    this.args = void 0;
    this.to = void 0;
    this.origin = config.origin;
    this.origin_public_key = config.origin_public_key;
    this.encrypted_egress = config.encrypted_egress;
    this.transaction_action = config.transaction_action;
    this.receiver = config.receiver;
    this.amount = config.amount;
    this.wasm_blob = config.wasm_blob;
    this.method = config.method;
    this.args = config.args;
    this.to = config.to;
  }

}

exports.Call = Call;

class Calls {
  constructor(config) {
    this.ops = void 0;
    this.ops = config.ops;
  }

}

exports.Calls = Calls;

class Outcome {
  constructor(config) {
    this.view_result_log = void 0;
    this.view_result = void 0;
    this.outcome_logs = void 0;
    this.outcome_receipt_ids = void 0;
    this.outcome_gas_burnt = void 0;
    this.outcome_token_burnt = void 0;
    this.outcome_executor_id = void 0;
    this.outcome_status = void 0;
    this.view_result_log = config.view_result_log;
    this.view_result = config.view_result;
    this.outcome_logs = config.outcome_logs;
    this.outcome_receipt_ids = config.outcome_receipt_ids;
    this.outcome_gas_burnt = config.outcome_gas_burnt;
    this.outcome_token_burnt = config.outcome_token_burnt;
    this.outcome_executor_id = config.outcome_executor_id;
    this.outcome_status = config.outcome_status;
  }

}

exports.Outcome = Outcome;

class Outcomes {
  constructor(config) {
    this.ops = void 0;
    this.state_root = void 0;
    this.ops = config.ops;
    this.state_root = config.state_root;
  }

}

exports.Outcomes = Outcomes;

class LocalMetadata {
  constructor(config) {
    this.shard_id = void 0;
    this.high_remote_block = void 0;
    this.high_local_block = void 0;
    this.shard_id = config.shard_id;
    this.high_remote_block = config.high_remote_block;
    this.high_local_block = config.high_local_block;
  }

}

exports.LocalMetadata = LocalMetadata;

class Block {
  constructor(config) {
    this.shard_id = void 0;
    this.block_number = void 0;
    this.calls = void 0;
    this.shard_id = config.shard_id;
    this.block_number = config.block_number;
    this.calls = config.calls;
  }

}

exports.Block = Block;

class Shard {
  constructor(config) {
    this.high_remote_call_index = void 0;
    this.high_local_call_index = void 0;
    this.high_remote_synced_block_index = void 0;
    this.high_remote_confirmed_block_index = void 0;
    this.high_remote_call_index = config.high_remote_call_index;
    this.high_local_call_index = config.high_local_call_index;
    this.high_remote_synced_block_index = config.high_remote_synced_block_index;
    this.high_remote_confirmed_block_index = config.high_remote_confirmed_block_index;
  }

}

exports.Shard = Shard;

class ShardMetadata {
  constructor(config) {
    this.shard_key = void 0;
    this.shard_members = void 0;
    this.beacon_index = void 0;
    this.threshold = void 0;
    this.shard_key = config.shard_key;
    this.shard_members = config.shard_members;
    this.beacon_index = config.beacon_index;
    this.threshold = config.threshold;
  }

}

exports.ShardMetadata = ShardMetadata;

class Contract {
  constructor(config) {
    this.home_shard = void 0;
    this.wasm_blob = void 0;
    this.metadata_cid = void 0;
    this.metadata = void 0;
    this.is_initial_state_empty = void 0;
    this.home_shard = config.home_shard;
    this.wasm_blob = config.wasm_blob;
    this.metadata_cid = (0, _util.stringToU8a)(config.metadata_cid);
    this.metadata = config.metadata;
    this.is_initial_state_empty = config.is_initial_state_empty;
  }

}

exports.Contract = Contract;

class ExecutionSummary {
  constructor(config) {
    this.high_local_execution_block = void 0;
    this.high_local_execution_block = config.high_local_execution_block;
  }

}

exports.ExecutionSummary = ExecutionSummary;
const blockSchema = new Map([[Block, {
  kind: 'struct',
  fields: [['shard_id', 'u32'], ['block_number', 'u32'], ['calls', {
    kind: 'option',
    type: ['u32']
  }]]
}]]);
exports.blockSchema = blockSchema;
const callSchema = new Map([[Call, {
  kind: 'struct',
  fields: [['origin', {
    kind: 'option',
    type: 'string'
  }], ['origin_public_key', {
    kind: 'option',
    type: ['u8', 32]
  }], ['encrypted_egress', 'u8'], ['transaction_action', 'string'], ['receiver', 'string'], ['amount', {
    kind: 'option',
    type: 'u128'
  }], ['wasm_blob', {
    kind: 'option',
    type: ['u8']
  }], ['method', {
    kind: 'option',
    type: 'string'
  }], ['args', {
    kind: 'option',
    type: 'string'
  }], ['to', {
    kind: 'option',
    type: 'string'
  }]]
}]]);
exports.callSchema = callSchema;
const callsSchema = new Map();
exports.callsSchema = callsSchema;
callsSchema.set(Calls, {
  kind: 'struct',
  fields: [['ops', [Call]]]
});
callsSchema.set(Call, callSchema.get(Call));
const outcomeSchema = new Map([[Outcome, {
  kind: 'struct',
  fields: [['view_result_log', ['string']], ['view_result', ['u8']], ['outcome_logs', ['string']], ['outcome_receipt_ids', [['u8', 32]]], ['outcome_gas_burnt', 'u64'], ['outcome_token_burnt', 'u128'], ['outcome_executor_id', 'String'], ['outcome_status', {
    kind: 'option',
    type: ['u8']
  }]]
}]]);
exports.outcomeSchema = outcomeSchema;
const outcomesSchema = new Map();
exports.outcomesSchema = outcomesSchema;
outcomesSchema.set(Outcomes, {
  kind: 'struct',
  fields: [['ops', [Outcome]], ['state_root', ['u8', 32]]]
});
outcomesSchema.set(Outcome, outcomeSchema.get(Outcome));
const contractSchema = new Map([[Contract, {
  kind: 'struct',
  fields: [['home_shard', 'u32'], ['wasm_blob', ['u8']], ['metadata_cid', ['u8', 46]], ['metadata', ['u8']], ['is_initial_state_empty', 'u8']]
}]]);
exports.contractSchema = contractSchema;
const shardMetadataSchema = new Map([[ShardMetadata, {
  kind: 'struct',
  fields: [['shard_key', ['u8', 32]], ['shard_members', ['string']], ['beacon_index', 'u32'], ['threshold', 'u32']]
}]]);
exports.shardMetadataSchema = shardMetadataSchema;
const shardSchema = new Map([[Shard, {
  kind: 'struct',
  fields: [['high_remote_call_index', 'u32'], ['high_local_call_index', 'u32'], ['high_remote_synced_block_index', 'u32'], ['high_remote_confirmed_block_index', 'u32']]
}]]);
exports.shardSchema = shardSchema;
const localMetadataSchema = new Map([[LocalMetadata, {
  kind: 'struct',
  fields: [['shard_id', ['u32']], ['high_remote_block', 'u32'], ['high_local_block', 'u32']]
}]]);
exports.localMetadataSchema = localMetadataSchema;
const executionSummarySchema = new Map([[ExecutionSummary, {
  kind: 'struct',
  fields: [['high_local_execution_block', 'u32']]
}]]); // ser

exports.executionSummarySchema = executionSummarySchema;

const buildCall = call => {
  const buf = (0, _borsh.serialize)(callSchema, call);
  return (0, _borsh.baseEncode)(buf);
};

exports.buildCall = buildCall;

const buildBlock = block => {
  const buf = (0, _borsh.serialize)(blockSchema, block);
  return (0, _borsh.baseEncode)(buf);
};

exports.buildBlock = buildBlock;

const buildContract = contract => {
  const buf = (0, _borsh.serialize)(contractSchema, contract);
  return (0, _borsh.baseEncode)(buf);
};

exports.buildContract = buildContract;

const buildCalls = calls => {
  if (calls.ops.length === 0) {
    return '';
  }

  const buf = (0, _borsh.serialize)(callsSchema, calls);
  return (0, _borsh.baseEncode)(buf);
};

exports.buildCalls = buildCalls;

const buildOutcome = outcome => {
  const buf = (0, _borsh.serialize)(outcomeSchema, outcome);
  return (0, _borsh.baseEncode)(buf);
};

exports.buildOutcome = buildOutcome;

const buildOutcomes = outcomes => {
  const buf = (0, _borsh.serialize)(outcomesSchema, outcomes);
  return (0, _borsh.baseEncode)(buf);
};

exports.buildOutcomes = buildOutcomes;

const buildShard = shard => {
  const buf = (0, _borsh.serialize)(shardSchema, shard);
  return (0, _borsh.baseEncode)(buf);
};

exports.buildShard = buildShard;

const buildShardMetadata = shardMetadata => {
  const buf = (0, _borsh.serialize)(shardMetadataSchema, shardMetadata);
  return (0, _borsh.baseEncode)(buf);
};

exports.buildShardMetadata = buildShardMetadata;

const buildLocalMetadata = a => {
  const buf = (0, _borsh.serialize)(localMetadataSchema, a);
  return (0, _borsh.baseEncode)(buf);
};

exports.buildLocalMetadata = buildLocalMetadata;

const buildExecutionSummary = a => {
  const buf = (0, _borsh.serialize)(executionSummarySchema, a);
  return (0, _borsh.baseEncode)(buf);
}; // de


exports.buildExecutionSummary = buildExecutionSummary;

const parseCall = buf => {
  const c = (0, _borsh.deserialize)(callSchema, Call, (0, _borsh.baseDecode)(buf));
  c.encrypted_egress = !!c.encrypted_egress;
  return c;
};

exports.parseCall = parseCall;

const parseCalls = buf => {
  if (buf.length === 0) {
    return {
      ops: []
    };
  }

  const cs = (0, _borsh.deserialize)(callsSchema, Calls, (0, _borsh.baseDecode)(buf));

  for (let i = 0; i < cs.ops.length; i++) {
    cs.ops[i].encrypted_egress = !!cs.ops[i].encrypted_egress;
  }

  return cs;
};

exports.parseCalls = parseCalls;

const parseOutcome = buf => {
  return (0, _borsh.deserialize)(outcomeSchema, Outcome, (0, _borsh.baseDecode)(buf));
};

exports.parseOutcome = parseOutcome;

const parseOutcomes = buf => {
  return (0, _borsh.deserialize)(outcomesSchema, Outcomes, (0, _borsh.baseDecode)(buf));
};

exports.parseOutcomes = parseOutcomes;

const parseBlock = buf => {
  return (0, _borsh.deserialize)(blockSchema, Block, (0, _borsh.baseDecode)(buf));
};

exports.parseBlock = parseBlock;

const parseContract = buf => {
  const c = (0, _borsh.deserialize)(contractSchema, Contract, (0, _borsh.baseDecode)(buf));
  c.is_initial_state_empty = !!c.is_initial_state_empty;
  return c;
};

exports.parseContract = parseContract;

const parseShard = buf => {
  return (0, _borsh.deserialize)(shardSchema, Shard, (0, _borsh.baseDecode)(buf));
};

exports.parseShard = parseShard;

const parseShardMetadata = buf => {
  return (0, _borsh.deserialize)(shardMetadataSchema, ShardMetadata, (0, _borsh.baseDecode)(buf));
};

exports.parseShardMetadata = parseShardMetadata;

const parseLocalMetadata = buf => {
  return (0, _borsh.deserialize)(localMetadataSchema, LocalMetadata, (0, _borsh.baseDecode)(buf));
};

exports.parseLocalMetadata = parseLocalMetadata;

const parseExecutionSummary = buf => {
  return (0, _borsh.deserialize)(executionSummarySchema, ExecutionSummary, (0, _borsh.baseDecode)(buf));
};

exports.parseExecutionSummary = parseExecutionSummary;