import BN from 'bn.js';
declare class Call {
    origin: string;
    origin_public_key: Uint8Array;
    encrypted_egress: boolean;
    transaction_action: 'create_account' | 'call' | 'transfer' | 'view_method_call' | 'deploy';
    receiver: string;
    amount: BN | null;
    wasm_blob: Uint8Array | null;
    method: string | null;
    args: string | null;
    to: string | null;
    constructor(config: {
        origin: string;
        origin_public_key: Uint8Array;
        encrypted_egress: boolean;
        transaction_action: 'create_account' | 'call' | 'transfer' | 'view_method_call' | 'deploy';
        receiver: string;
        amount: BN | null;
        wasm_blob: Uint8Array | null;
        method: string | null;
        args: string | null;
        to: string | null;
    });
}
declare class Calls {
    ops: Call[];
    constructor(config: {
        ops: Call[];
    });
}
declare class Outcome {
    view_result_log: string[];
    view_result: Uint8Array;
    outcome_logs: string[];
    outcome_receipt_ids: Uint8Array[];
    outcome_gas_burnt: BN;
    outcome_token_burnt: BN;
    outcome_executor_id: string;
    outcome_status: Uint8Array | null;
    constructor(config: {
        view_result_log: string[];
        view_result: Uint8Array;
        outcome_logs: string[];
        outcome_receipt_ids: Uint8Array[];
        outcome_gas_burnt: BN;
        outcome_token_burnt: BN;
        outcome_executor_id: string;
        outcome_status: Uint8Array | null;
    });
}
declare class Outcomes {
    ops: Outcome[];
    state_root: Uint8Array;
    constructor(config: {
        ops: Outcome[];
        state_root: Uint8Array;
    });
}
declare class LocalMetadata {
    shard_id: number[];
    high_remote_block: number;
    high_local_block: number;
    constructor(config: {
        shard_id: number[];
        high_remote_block: number;
        high_local_block: number;
    });
}
declare class Block {
    shard_id: number;
    block_number: number;
    calls: number[];
    constructor(config: {
        shard_id: number;
        block_number: number;
        calls: number[];
    });
}
declare class Shard {
    high_remote_call_index: number;
    high_local_call_index: number;
    high_remote_synced_block_index: number;
    high_remote_confirmed_block_index: number;
    constructor(config: {
        high_remote_call_index: number;
        high_local_call_index: number;
        high_remote_synced_block_index: number;
        high_remote_confirmed_block_index: number;
    });
}
declare class ShardMetadata {
    shard_key: Uint8Array;
    shard_members: string[];
    beacon_index: number;
    threshold: number;
    constructor(config: {
        shard_key: Uint8Array;
        shard_members: string[];
        beacon_index: number;
        threshold: number;
    });
}
declare class Contract {
    home_shard: number;
    wasm_blob: Uint8Array;
    metadata_cid: Uint8Array;
    metadata: Uint8Array;
    is_initial_state_empty: boolean;
    constructor(config: {
        home_shard: number;
        wasm_blob: Uint8Array;
        metadata_cid: string;
        metadata: Uint8Array;
        is_initial_state_empty: boolean;
    });
}
declare class ExecutionSummary {
    high_local_execution_block: number;
    constructor(config: {
        high_local_execution_block: number;
    });
}
declare const blockSchema: Map<typeof Block, {
    kind: string;
    fields: (string | {
        kind: string;
        type: string[];
    })[][];
}>;
declare const callSchema: Map<typeof Call, {
    kind: string;
    fields: ((string | {
        kind: string;
        type: string;
    })[] | (string | {
        kind: string;
        type: (string | number)[];
    })[])[];
}>;
declare const callsSchema: Map<any, any>;
declare const outcomeSchema: Map<typeof Outcome, {
    kind: string;
    fields: ((string | string[])[] | (string | (string | number)[][])[] | (string | {
        kind: string;
        type: string[];
    })[])[];
}>;
declare const outcomesSchema: Map<any, any>;
declare const contractSchema: Map<typeof Contract, {
    kind: string;
    fields: (string | (string | number)[])[][];
}>;
declare const shardMetadataSchema: Map<typeof ShardMetadata, {
    kind: string;
    fields: (string | (string | number)[])[][];
}>;
declare const shardSchema: Map<typeof Shard, {
    kind: string;
    fields: string[][];
}>;
declare const localMetadataSchema: Map<typeof LocalMetadata, {
    kind: string;
    fields: (string | string[])[][];
}>;
declare const executionSummarySchema: Map<typeof ExecutionSummary, {
    kind: string;
    fields: string[][];
}>;
declare const buildCall: (call: Call) => string;
declare const buildBlock: (block: Block) => string;
declare const buildContract: (contract: Contract) => string;
declare const buildCalls: (calls: Calls) => string;
declare const buildOutcome: (outcome: Outcome) => string;
declare const buildOutcomes: (outcomes: Outcomes) => string;
declare const buildShard: (shard: Shard) => string;
declare const buildShardMetadata: (shardMetadata: ShardMetadata) => string;
declare const buildLocalMetadata: (a: LocalMetadata) => string;
declare const buildExecutionSummary: (a: ExecutionSummary) => string;
declare const parseCall: (buf: string) => Call;
declare const parseCalls: (buf: string) => Calls;
declare const parseOutcome: (buf: string) => Outcome;
declare const parseOutcomes: (buf: string) => Outcomes;
declare const parseBlock: (buf: string) => Block;
declare const parseContract: (buf: string) => Contract;
declare const parseShard: (buf: string) => Shard;
declare const parseShardMetadata: (buf: string) => ShardMetadata;
declare const parseLocalMetadata: (buf: string) => LocalMetadata;
declare const parseExecutionSummary: (buf: string) => ExecutionSummary;
export { Call, callSchema, buildCall, parseCall, Calls, callsSchema, buildCalls, parseCalls, Outcome, outcomeSchema, buildOutcome, parseOutcome, Outcomes, outcomesSchema, buildOutcomes, parseOutcomes, Block, blockSchema, buildBlock, parseBlock, Contract, contractSchema, buildContract, parseContract, Shard, shardSchema, buildShard, parseShard, ShardMetadata, shardMetadataSchema, buildShardMetadata, parseShardMetadata, LocalMetadata, localMetadataSchema, buildLocalMetadata, parseLocalMetadata, ExecutionSummary, executionSummarySchema, buildExecutionSummary, parseExecutionSummary };
