"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _utilCrypto = require("@polkadot/util-crypto");

var _bn = _interopRequireDefault(require("bn.js"));

var _dotenv = _interopRequireDefault(require("dotenv"));

var _secretRegistry = require("@skyekiwi/secret-registry");

var _util = require("@skyekiwi/util");

var _index = require("./index.cjs");

// Copyright 2021-2022 @skyekiwi/s-contract authors & contributors
// SPDX-License-Identifier: Apache-2.0
_dotenv.default.config();

describe('@skyekiwi/s-contract/contract', function () {
  const mnemonic = process.env.SEED_PHRASE;

  if (!(0, _utilCrypto.mnemonicValidate)(mnemonic)) {
    throw new Error('mnemonic failed to read - e2e.spec.ts');
  }

  const registry = new _secretRegistry.SecretRegistry(mnemonic, {});
  afterAll(async () => {
    await registry.disconnect();
  });
  test('upstream/downstream contract without initial state', async () => {
    const calls = new _index.Calls({
      ops: []
    });

    const contract = _index.ContractController.intoSecretContract(calls, new Uint8Array([0x1, 0x2, 0x3, 0x4]));

    const result = await _index.ContractController.upstream(registry, contract);
    expect(result).not.toBeNull();
    const downstreamedContract = await _index.ContractController.downstream(registry, result);
    expect((0, _util.u8aToHex)(downstreamedContract.initialState)).toEqual((0, _util.u8aToHex)(contract.initialState));
    expect(downstreamedContract.secretId).toEqual(result);
    expect(downstreamedContract.wasmBlob).toEqual(contract.wasmBlob); // await registry.disconnect();
  });
  test('upstream/downstream contract with initial state', async () => {
    /* eslint-disable sort-keys, camelcase */
    const call1 = new _index.Call({
      origin: '5GrwvaEF5zXb26Fz9rcQpDWS57CtERHpNehXCPcNoHGKutQY',
      origin_public_key: new Uint8Array([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]),
      encrypted_egress: false,
      transaction_action: 'create_account',
      receiver: 'test',
      amount: new _bn.default(0x100, 16),
      wasm_blob: new Uint8Array([1, 2, 3]),
      method: undefined,
      args: undefined,
      to: undefined
    });
    const call2 = new _index.Call({
      origin: '5GrwvaEF5zXb26Fz9rcQpDWS57CtERHpNehXCPcNoHGKutQY',
      origin_public_key: new Uint8Array([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]),
      encrypted_egress: false,
      transaction_action: 'create_account',
      receiver: 'test2',
      amount: new _bn.default(0x100, 16),
      wasm_blob: new Uint8Array([1, 2, 3]),
      method: undefined,
      args: undefined,
      to: undefined
    });
    /* eslint-enable sort-keys, camelcase */

    const calls = new _index.Calls({
      ops: [call1, call2]
    });

    const contract = _index.ContractController.intoSecretContract(calls, new Uint8Array([0x1, 0x2, 0x3, 0x4]));

    const result = await _index.ContractController.upstream(registry, contract);
    expect(result).not.toBeNull();
    const downstreamedContract = await _index.ContractController.downstream(registry, result);
    expect((0, _util.u8aToHex)(downstreamedContract.initialState)).toEqual((0, _util.u8aToHex)(contract.initialState));
    expect(downstreamedContract.secretId).toEqual(result);
    expect((0, _util.u8aToHex)(downstreamedContract.wasmBlob)).toEqual((0, _util.u8aToHex)(contract.wasmBlob));
    await registry.disconnect();
  });
});