"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Indexer = void 0;

var _classPrivateFieldLooseBase2 = _interopRequireDefault(require("@babel/runtime/helpers/classPrivateFieldLooseBase"));

var _classPrivateFieldLooseKey2 = _interopRequireDefault(require("@babel/runtime/helpers/classPrivateFieldLooseKey"));

var _level = _interopRequireDefault(require("level"));

var _ipfs = require("@skyekiwi/ipfs");

var _util = require("@skyekiwi/util");

var _borsh = require("./borsh.cjs");

var _index = require("./index.cjs");

// Copyright 2021-2022 @skyekiwi/s-contract authors & contributors
// SPDX-License-Identifier: Apache-2.0
var _db = /*#__PURE__*/(0, _classPrivateFieldLooseKey2.default)("db");

var _ops = /*#__PURE__*/(0, _classPrivateFieldLooseKey2.default)("ops");

/* eslint-disable sort-keys, camelcase, @typescript-eslint/ban-ts-comment */
class Indexer {
  constructor() {
    Object.defineProperty(this, _db, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _ops, {
      writable: true,
      value: void 0
    });
  }

  init() {
    (0, _classPrivateFieldLooseBase2.default)(this, _ops)[_ops] = [];
    (0, _classPrivateFieldLooseBase2.default)(this, _db)[_db] = (0, _level.default)('local');
  }

  async initializeLocalMetadata() {
    const localMetadata = new _index.LocalMetadata({
      shard_id: [0],
      high_remote_block: 0,
      high_local_block: 0
    });

    (0, _classPrivateFieldLooseBase2.default)(this, _ops)[_ops].push(_index.Storage.writeMetadata(localMetadata));

    await _index.Storage.writeAll((0, _classPrivateFieldLooseBase2.default)(this, _db)[_db], (0, _classPrivateFieldLooseBase2.default)(this, _ops)[_ops]);
    (0, _classPrivateFieldLooseBase2.default)(this, _ops)[_ops] = [];
  }

  async fetchAll(api) {
    const logger = (0, _util.getLogger)('indexer.fetchAll');
    const localMetadata = await _index.Storage.getMetadata((0, _classPrivateFieldLooseBase2.default)(this, _db)[_db]);
    const highLocalBlock = localMetadata.high_local_block ? localMetadata.high_local_block : 0;
    logger.info(`highest local block at ${localMetadata.high_local_block}`);
    let currentBlockNumber = highLocalBlock + 1;

    while (true) {
      logger.debug(`fetching all info from block# ${currentBlockNumber}`);

      for (const shardId of localMetadata.shard_id) {
        await this.fetchCalls(api, shardId, currentBlockNumber);
      }

      currentBlockNumber++;
      const currentHighBlockNumber = Number((await api.query.system.number()).toJSON());

      if (isNaN(currentHighBlockNumber) || currentBlockNumber >= currentHighBlockNumber) {
        logger.info(`all catchuped ... for now at block# ${currentHighBlockNumber}`);
        break;
      }
    }
  }

  async fetchCalls(api, shardId, blockNumber) {
    const logger = (0, _util.getLogger)('indexer.fetchCalls');
    const calls = (await api.query.sContract.callHistory(shardId, blockNumber)).toJSON(); // 1. build blocks

    const block = new _index.Block({
      shard_id: shardId,
      block_number: blockNumber,
      calls: calls
    });

    (0, _classPrivateFieldLooseBase2.default)(this, _ops)[_ops].push(_index.Storage.writeBlockRecord(shardId, blockNumber, block));

    if (!calls) return;
    logger.info(`block import complete at block# ${blockNumber}`); // 2. build calls

    for (const call of calls) {
      const callContentRaw = (await api.query.sContract.callRecord(shardId, call)).toJSON();
      const callContent = callContentRaw; // logger.debug(callContent);

      console.log(callContent[0]);
      console.log(callContent[1]); // const callRecord = parseCall(callContent[0])
      // this.#ops.push(Storage.writeCallRecord(shardId, blockNumber, call, callRecord));
    }

    logger.info(`call import complete at block# ${blockNumber}`); // 3. build contracts

    const blockHash = (await api.query.system.blockHash(blockNumber)).toJSON();
    const events = (await api.query.system.events.at(blockHash)).toHuman();
    events.map(async evt => {
      if (evt.event) {
        if (evt.event.method === 'SecretContractRegistered') {
          const secretId = Number(evt.event.data[0]);
          const homeShard = (await api.query.secrets.homeShard(secretId)).toJSON();
          const wasmCIDRaw = (await api.query.secrets.wasmBlob(secretId)).toJSON();
          const wasmCID = (0, _util.u8aToString)((0, _util.hexToU8a)(wasmCIDRaw.substring(2)));
          console.log(wasmCID);
          const ipfs = new _ipfs.IPFS();
          const wasmBlob = (0, _util.hexToU8a)(await ipfs.cat(wasmCID));
          const metadataCIDRaw = (await api.query.secrets.metadata(secretId)).toJSON();
          const metadataCID = (0, _util.u8aToString)((0, _util.hexToU8a)(metadataCIDRaw.substring(2)));

          if (metadataCID === '0000000000000000000000000000000000000000000000') {
            const metadata = (0, _util.hexToU8a)(await ipfs.cat(metadataCID));
            const contract = new _index.Contract({
              home_shard: homeShard,
              wasm_blob: wasmBlob,
              metadata_cid: metadataCID,
              metadata: metadata,
              is_initial_state_empty: false
            });

            (0, _classPrivateFieldLooseBase2.default)(this, _ops)[_ops].push(_index.Storage.writeContractRecord(secretId, contract));
          } else {
            const contract = new _index.Contract({
              home_shard: homeShard,
              wasm_blob: wasmBlob,
              metadata_cid: metadataCID,
              metadata: null,
              is_initial_state_empty: true
            });

            (0, _classPrivateFieldLooseBase2.default)(this, _ops)[_ops].push(_index.Storage.writeContractRecord(secretId, contract));
          }
        }
      }
    });
    logger.info(`contract import complete at block# ${blockNumber}`);
  }

  async fetchOnce(api, address) {
    const localMetadata = await _index.Storage.getMetadata((0, _classPrivateFieldLooseBase2.default)(this, _db)[_db]);

    for (const shard of localMetadata.shard_id) {
      const maybeMembers = await api.query.registry.shardMembers(shard);
      const maybeBeaconIndex = await api.query.registry.beaconIndex(shard, address);
      const maybeThreshold = await api.query.parentchain.shardConfirmationThreshold(shard);
      const members = maybeMembers.toJSON();
      const beaconIndex = Number(maybeBeaconIndex.toString());
      const threshold = Number(maybeThreshold.toString());
      let shardInfo;

      try {
        shardInfo = await _index.Storage.getShardMetadataRecord((0, _classPrivateFieldLooseBase2.default)(this, _db)[_db], shard); // shard has been recorded in the system
        // Updating ...

        shardInfo.shard_members = members;
        shardInfo.beacon_index = beaconIndex;
        shardInfo.threshold = threshold || threshold === 0 ? 1 : threshold;
      } catch (e) {
        // shard has not been recorded in the system
        // Recording ...
        shardInfo = new _borsh.ShardMetadata({
          shard_key: new Uint8Array([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]),
          shard_members: members,
          beacon_index: beaconIndex,
          threshold: threshold
        });
      }

      (0, _classPrivateFieldLooseBase2.default)(this, _ops)[_ops].push(_index.Storage.writeShardMetadataRecord(shard, shardInfo));
    }
  }

  async writeAll() {
    await _index.Storage.writeAll((0, _classPrivateFieldLooseBase2.default)(this, _db)[_db], (0, _classPrivateFieldLooseBase2.default)(this, _ops)[_ops]);
  }

}

exports.Indexer = Indexer;