"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ShardManager = void 0;

var _classPrivateFieldLooseBase2 = _interopRequireDefault(require("@babel/runtime/helpers/classPrivateFieldLooseBase"));

var _classPrivateFieldLooseKey2 = _interopRequireDefault(require("@babel/runtime/helpers/classPrivateFieldLooseKey"));

var _api2 = require("@polkadot/api");

var _keyring2 = require("@polkadot/keyring");

var _wasmCrypto = require("@polkadot/wasm-crypto");

var _tweetnacl = require("tweetnacl");

var _crypto = require("@skyekiwi/crypto");

var _util = require("@skyekiwi/util");

// Copyright 2021-2022 @skyekiwi/s-contract authors & contributors
// SPDX-License-Identifier: Apache-2.0
var _provider = /*#__PURE__*/(0, _classPrivateFieldLooseKey2.default)("provider");

var _keyring = /*#__PURE__*/(0, _classPrivateFieldLooseKey2.default)("keyring");

var _api = /*#__PURE__*/(0, _classPrivateFieldLooseKey2.default)("api");

var _key = /*#__PURE__*/(0, _classPrivateFieldLooseKey2.default)("key");

var _shards = /*#__PURE__*/(0, _classPrivateFieldLooseKey2.default)("shards");

var _shardMembers = /*#__PURE__*/(0, _classPrivateFieldLooseKey2.default)("shardMembers");

var _beaconIndex = /*#__PURE__*/(0, _classPrivateFieldLooseKey2.default)("beaconIndex");

var _threshold = /*#__PURE__*/(0, _classPrivateFieldLooseKey2.default)("threshold");

class ShardManager {
  constructor(runningShards, key) {
    Object.defineProperty(this, _provider, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _keyring, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _api, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _key, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _shards, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _shardMembers, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _beaconIndex, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _threshold, {
      writable: true,
      value: void 0
    });
    (0, _classPrivateFieldLooseBase2.default)(this, _shards)[_shards] = runningShards;
    (0, _classPrivateFieldLooseBase2.default)(this, _key)[_key] = key || (0, _tweetnacl.randomBytes)(32);
    (0, _classPrivateFieldLooseBase2.default)(this, _shardMembers)[_shardMembers] = {};
    (0, _classPrivateFieldLooseBase2.default)(this, _beaconIndex)[_beaconIndex] = {};
    (0, _classPrivateFieldLooseBase2.default)(this, _threshold)[_threshold] = {};
  }

  async init() {
    const seed = process.env.TEST_SEED_PHRASE;

    if (!seed) {
      throw new Error('seed phrase not found');
    }

    await (0, _wasmCrypto.waitReady)();
    (0, _classPrivateFieldLooseBase2.default)(this, _provider)[_provider] = new _api2.WsProvider('ws://localhost:9944'); // this.#provider = new WsProvider('wss://staging.rpc.skye.kiwi');

    (0, _classPrivateFieldLooseBase2.default)(this, _api)[_api] = await _api2.ApiPromise.create({
      provider: (0, _classPrivateFieldLooseBase2.default)(this, _provider)[_provider]
    });
    (0, _classPrivateFieldLooseBase2.default)(this, _keyring)[_keyring] = new _keyring2.Keyring({
      type: 'sr25519'
    }).addFromUri(seed);
  }

  async disconnect() {
    await (0, _classPrivateFieldLooseBase2.default)(this, _provider)[_provider].disconnect();
  }

  async maybeRegisterSecretKeeper(blockNumber) {
    const allExtrinsics = [];
    const maybeExpiration = await (0, _classPrivateFieldLooseBase2.default)(this, _api)[_api].query.registry.expiration((0, _classPrivateFieldLooseBase2.default)(this, _keyring)[_keyring].address);
    const expiration = Number(maybeExpiration.toString());

    if (isNaN(expiration) || expiration - 10 < blockNumber) {
      // not previously registered
      allExtrinsics.push((0, _classPrivateFieldLooseBase2.default)(this, _api)[_api].tx.registry.registerSecretKeeper((0, _util.u8aToHex)(_crypto.AsymmetricEncryption.getPublicKey((0, _classPrivateFieldLooseBase2.default)(this, _key)[_key])), '0x0000'));

      for (const shard of (0, _classPrivateFieldLooseBase2.default)(this, _shards)[_shards]) {
        allExtrinsics.push((0, _classPrivateFieldLooseBase2.default)(this, _api)[_api].tx.registry.registerRunningShard(shard));
      }

      const all = (0, _classPrivateFieldLooseBase2.default)(this, _api)[_api].tx.utility.batch(allExtrinsics);

      await (0, _util.sendTx)(all, (0, _classPrivateFieldLooseBase2.default)(this, _keyring)[_keyring]);
    }
  }

  maybeSubmitExecutionReport(blockNumber) {
    for (const shard of (0, _classPrivateFieldLooseBase2.default)(this, _shards)[_shards]) {
      console.log(this.beaconIsTurn(blockNumber, shard));

      if (this.beaconIsTurn(blockNumber, shard)) {
        console.log('Submitting Tx');
      }
    }
  }

  beaconIsTurn(blockNumber, shard) {
    const beaconIndex = (0, _classPrivateFieldLooseBase2.default)(this, _beaconIndex)[_beaconIndex][shard];

    const threshold = (0, _classPrivateFieldLooseBase2.default)(this, _threshold)[_threshold][shard];

    const beaconCount = (0, _classPrivateFieldLooseBase2.default)(this, _shardMembers)[_shardMembers][shard].length;

    console.log(beaconIndex, threshold, beaconCount); // 1 2 3 4 5 6 7 8 9

    return threshold >= beaconCount || // _ X X X _ _ _ _ _
    blockNumber % beaconCount <= beaconIndex && beaconIndex <= blockNumber % beaconCount + threshold - 1 || // X X _ _ _ _ _ _ X
    blockNumber % beaconCount + threshold - 1 > beaconCount && (beaconCount - (blockNumber % beaconCount + threshold - 1) % beaconCount <= beaconIndex || beaconIndex <= blockNumber % beaconCount + threshold - 1 - beaconCount);
  }

}

exports.ShardManager = ShardManager;