import _classPrivateFieldLooseBase from "@babel/runtime/helpers/esm/classPrivateFieldLooseBase";
import _classPrivateFieldLooseKey from "@babel/runtime/helpers/esm/classPrivateFieldLooseKey";
// Copyright 2021-2022 @skyekiwi/s-contract authors & contributors
// SPDX-License-Identifier: Apache-2.0
import { ApiPromise, WsProvider } from '@polkadot/api';
import { Keyring } from '@polkadot/keyring';
import { waitReady } from '@polkadot/wasm-crypto';
import { randomBytes } from 'tweetnacl';
import { AsymmetricEncryption } from '@skyekiwi/crypto';
import { sendTx, u8aToHex } from '@skyekiwi/util';

var _provider = /*#__PURE__*/_classPrivateFieldLooseKey("provider");

var _keyring = /*#__PURE__*/_classPrivateFieldLooseKey("keyring");

var _api = /*#__PURE__*/_classPrivateFieldLooseKey("api");

var _key = /*#__PURE__*/_classPrivateFieldLooseKey("key");

var _shards = /*#__PURE__*/_classPrivateFieldLooseKey("shards");

var _shardMembers = /*#__PURE__*/_classPrivateFieldLooseKey("shardMembers");

var _beaconIndex = /*#__PURE__*/_classPrivateFieldLooseKey("beaconIndex");

var _threshold = /*#__PURE__*/_classPrivateFieldLooseKey("threshold");

export class ShardManager {
  constructor(runningShards, key) {
    Object.defineProperty(this, _provider, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _keyring, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _api, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _key, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _shards, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _shardMembers, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _beaconIndex, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _threshold, {
      writable: true,
      value: void 0
    });
    _classPrivateFieldLooseBase(this, _shards)[_shards] = runningShards;
    _classPrivateFieldLooseBase(this, _key)[_key] = key || randomBytes(32);
    _classPrivateFieldLooseBase(this, _shardMembers)[_shardMembers] = {};
    _classPrivateFieldLooseBase(this, _beaconIndex)[_beaconIndex] = {};
    _classPrivateFieldLooseBase(this, _threshold)[_threshold] = {};
  }

  async init() {
    const seed = process.env.TEST_SEED_PHRASE;

    if (!seed) {
      throw new Error('seed phrase not found');
    }

    await waitReady();
    _classPrivateFieldLooseBase(this, _provider)[_provider] = new WsProvider('ws://localhost:9944'); // this.#provider = new WsProvider('wss://staging.rpc.skye.kiwi');

    _classPrivateFieldLooseBase(this, _api)[_api] = await ApiPromise.create({
      provider: _classPrivateFieldLooseBase(this, _provider)[_provider]
    });
    _classPrivateFieldLooseBase(this, _keyring)[_keyring] = new Keyring({
      type: 'sr25519'
    }).addFromUri(seed);
  }

  async disconnect() {
    await _classPrivateFieldLooseBase(this, _provider)[_provider].disconnect();
  }

  async maybeRegisterSecretKeeper(blockNumber) {
    const allExtrinsics = [];
    const maybeExpiration = await _classPrivateFieldLooseBase(this, _api)[_api].query.registry.expiration(_classPrivateFieldLooseBase(this, _keyring)[_keyring].address);
    const expiration = Number(maybeExpiration.toString());

    if (isNaN(expiration) || expiration - 10 < blockNumber) {
      // not previously registered
      allExtrinsics.push(_classPrivateFieldLooseBase(this, _api)[_api].tx.registry.registerSecretKeeper(u8aToHex(AsymmetricEncryption.getPublicKey(_classPrivateFieldLooseBase(this, _key)[_key])), '0x0000'));

      for (const shard of _classPrivateFieldLooseBase(this, _shards)[_shards]) {
        allExtrinsics.push(_classPrivateFieldLooseBase(this, _api)[_api].tx.registry.registerRunningShard(shard));
      }

      const all = _classPrivateFieldLooseBase(this, _api)[_api].tx.utility.batch(allExtrinsics);

      await sendTx(all, _classPrivateFieldLooseBase(this, _keyring)[_keyring]);
    }
  }

  maybeSubmitExecutionReport(blockNumber) {
    for (const shard of _classPrivateFieldLooseBase(this, _shards)[_shards]) {
      console.log(this.beaconIsTurn(blockNumber, shard));

      if (this.beaconIsTurn(blockNumber, shard)) {
        console.log('Submitting Tx');
      }
    }
  }

  beaconIsTurn(blockNumber, shard) {
    const beaconIndex = _classPrivateFieldLooseBase(this, _beaconIndex)[_beaconIndex][shard];

    const threshold = _classPrivateFieldLooseBase(this, _threshold)[_threshold][shard];

    const beaconCount = _classPrivateFieldLooseBase(this, _shardMembers)[_shardMembers][shard].length;

    console.log(beaconIndex, threshold, beaconCount); // 1 2 3 4 5 6 7 8 9

    return threshold >= beaconCount || // _ X X X _ _ _ _ _
    blockNumber % beaconCount <= beaconIndex && beaconIndex <= blockNumber % beaconCount + threshold - 1 || // X X _ _ _ _ _ _ X
    blockNumber % beaconCount + threshold - 1 > beaconCount && (beaconCount - (blockNumber % beaconCount + threshold - 1) % beaconCount <= beaconIndex || beaconIndex <= blockNumber % beaconCount + threshold - 1 - beaconCount);
  }

}