"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Subscriber = void 0;

var _classPrivateFieldLooseBase2 = _interopRequireDefault(require("@babel/runtime/helpers/classPrivateFieldLooseBase"));

var _classPrivateFieldLooseKey2 = _interopRequireDefault(require("@babel/runtime/helpers/classPrivateFieldLooseKey"));

var _ipfs = require("@skyekiwi/ipfs");

var _util = require("@skyekiwi/util");

var _index = require("./index.cjs");

// Copyright 2021-2022 @skyekiwi/s-contract authors & contributors
// SPDX-License-Identifier: Apache-2.0
var _ops = /*#__PURE__*/(0, _classPrivateFieldLooseKey2.default)("ops");

/* eslint-disable sort-keys, camelcase, @typescript-eslint/ban-ts-comment */
class Subscriber {
  constructor() {
    Object.defineProperty(this, _ops, {
      writable: true,
      value: void 0
    });
  }

  init() {
    (0, _classPrivateFieldLooseBase2.default)(this, _ops)[_ops] = [];
  }

  async subscribeNewBlock(api, newBlockHook, newContractDeployHook, newCallsHook) {
    await api.rpc.chain.subscribeNewHeads(async latestHeader => {
      // new block hook comes first
      await newBlockHook(latestHeader.number.toNumber());
      await this.subscribeNewContractsDeployed(api, latestHeader.hash.toHex(), newContractDeployHook);
      await this.subscribeNewContractCallToShards(api, latestHeader.number.toNumber(), newCallsHook);
    });
  }

  async subscribeNewContractsDeployed(api, blockHash, callback) {
    // const contractIds = [];
    const events = (await api.query.system.events.at(blockHash)).toHuman();
    events.map(async evt => {
      if (evt.event) {
        if (evt.event.method === 'SecretContractRegistered') {
          const secretId = Number(evt.event.data[0]);
          const homeShard = (await api.query.secrets.homeShard(secretId)).toJSON();
          const wasmCIDRaw = (await api.query.secrets.wasmBlob(secretId)).toJSON();
          const wasmCID = (0, _util.u8aToString)((0, _util.hexToU8a)(wasmCIDRaw.substring(2)));
          console.log(wasmCID);
          const ipfs = new _ipfs.IPFS();
          const wasmBlob = (0, _util.hexToU8a)(await ipfs.cat(wasmCID));
          const metadataCIDRaw = (await api.query.secrets.metadata(secretId)).toJSON();
          const metadataCID = (0, _util.u8aToString)((0, _util.hexToU8a)(metadataCIDRaw.substring(2)));

          if (metadataCID === '0000000000000000000000000000000000000000000000') {
            const metadata = (0, _util.hexToU8a)(await ipfs.cat(metadataCID));
            const contract = new _index.Contract({
              home_shard: homeShard,
              wasm_blob: wasmBlob,
              metadata_cid: metadataCID,
              metadata: metadata,
              is_initial_state_empty: false
            });

            (0, _classPrivateFieldLooseBase2.default)(this, _ops)[_ops].push(_index.Storage.writeContractRecord(secretId, contract));
          } else {
            const contract = new _index.Contract({
              home_shard: homeShard,
              wasm_blob: wasmBlob,
              metadata_cid: metadataCID,
              metadata: null,
              is_initial_state_empty: true
            });

            (0, _classPrivateFieldLooseBase2.default)(this, _ops)[_ops].push(_index.Storage.writeContractRecord(secretId, contract));
          }
        }
      }
    });
    callback([]);
  }

  async subscribeNewContractCallToShards(api, blockNumber, callback) {
    const calls = await api.query.sContract.callHistory(0, blockNumber);
    callback(calls.toJSON());
  }

}

exports.Subscriber = Subscriber;