import _classPrivateFieldLooseBase from "@babel/runtime/helpers/esm/classPrivateFieldLooseBase";
import _classPrivateFieldLooseKey from "@babel/runtime/helpers/esm/classPrivateFieldLooseKey";
// Copyright 2021-2022 @skyekiwi/s-contract authors & contributors
// SPDX-License-Identifier: Apache-2.0
import { IPFS } from '@skyekiwi/ipfs';
import { hexToU8a, u8aToString } from '@skyekiwi/util';
import { Contract, Storage } from "./index.js";
/* eslint-disable sort-keys, camelcase, @typescript-eslint/ban-ts-comment */

var _ops = /*#__PURE__*/_classPrivateFieldLooseKey("ops");

export class Subscriber {
  constructor() {
    Object.defineProperty(this, _ops, {
      writable: true,
      value: void 0
    });
  }

  init() {
    _classPrivateFieldLooseBase(this, _ops)[_ops] = [];
  }

  async subscribeNewBlock(api, newBlockHook, newContractDeployHook, newCallsHook) {
    await api.rpc.chain.subscribeNewHeads(async latestHeader => {
      // new block hook comes first
      await newBlockHook(latestHeader.number.toNumber());
      await this.subscribeNewContractsDeployed(api, latestHeader.hash.toHex(), newContractDeployHook);
      await this.subscribeNewContractCallToShards(api, latestHeader.number.toNumber(), newCallsHook);
    });
  }

  async subscribeNewContractsDeployed(api, blockHash, callback) {
    // const contractIds = [];
    const events = (await api.query.system.events.at(blockHash)).toHuman();
    events.map(async evt => {
      if (evt.event) {
        if (evt.event.method === 'SecretContractRegistered') {
          const secretId = Number(evt.event.data[0]);
          const homeShard = (await api.query.secrets.homeShard(secretId)).toJSON();
          const wasmCIDRaw = (await api.query.secrets.wasmBlob(secretId)).toJSON();
          const wasmCID = u8aToString(hexToU8a(wasmCIDRaw.substring(2)));
          console.log(wasmCID);
          const ipfs = new IPFS();
          const wasmBlob = hexToU8a(await ipfs.cat(wasmCID));
          const metadataCIDRaw = (await api.query.secrets.metadata(secretId)).toJSON();
          const metadataCID = u8aToString(hexToU8a(metadataCIDRaw.substring(2)));

          if (metadataCID === '0000000000000000000000000000000000000000000000') {
            const metadata = hexToU8a(await ipfs.cat(metadataCID));
            const contract = new Contract({
              home_shard: homeShard,
              wasm_blob: wasmBlob,
              metadata_cid: metadataCID,
              metadata: metadata,
              is_initial_state_empty: false
            });

            _classPrivateFieldLooseBase(this, _ops)[_ops].push(Storage.writeContractRecord(secretId, contract));
          } else {
            const contract = new Contract({
              home_shard: homeShard,
              wasm_blob: wasmBlob,
              metadata_cid: metadataCID,
              metadata: null,
              is_initial_state_empty: true
            });

            _classPrivateFieldLooseBase(this, _ops)[_ops].push(Storage.writeContractRecord(secretId, contract));
          }
        }
      }
    });
    callback([]);
  }

  async subscribeNewContractCallToShards(api, blockNumber, callback) {
    const calls = await api.query.sContract.callHistory(0, blockNumber);
    callback(calls.toJSON());
  }

}